package org.vraptor.atx;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.objectweb.asm.ClassReader;

import java.io.*;
import java.util.ArrayList;
import java.util.List;

/**
 * Goal which translates annotations into a new vraptor xml file.
 * 
 * @goal translate
 * @phase compile
 * @since 1.0
 */
public class AnnotationsToXmlMojo extends AbstractMojo {

	/**
	 * Location of the file.
	 * 
	 * @parameter expression="${project.build.directory}/classes"
	 * @required
	 */
	private File outputDirectory;

	public void execute() throws MojoExecutionException {
		
		File f = outputDirectory;
		
		if (!f.exists()) {
			f.mkdirs();
		}

		getLog().debug("Base directory is " + f.getAbsolutePath());
		List<String> components = new ArrayList<String>();
		searchInside(f, "", components);

		try {
			FileWriter fw = new FileWriter(new File(f,"vraptor-vatx.xml"));
			PrintWriter writer = new PrintWriter(fw);
			writer.println("<vraptor>");
			for (String component : components) {
				writer.printf("\t<component>%s</component>\n", component);
			}
			writer.println("\t<autoDiscover>false</autoDiscover>");
			writer.println("</vraptor>");
			writer.close();
			fw.close();
		} catch (IOException ex) {
			throw new MojoExecutionException(
					"Unable to write vraptor xml file.", ex);
		}

	}

	private void searchInside(File f, String current, List<String> list)
			throws MojoExecutionException {
		getLog().debug("Searching " + f.getAbsolutePath() + " : " + current);
		for (File child : f.listFiles()) {
			if (child.isDirectory()) {
				searchInside(child, current + child.getName() + ".", list);
			} else {
				if (child.getName().endsWith(".class")) {
					try {
						FileInputStream is = new FileInputStream(child);
						getLog().debug("Processing " + child.getAbsolutePath());
						ClassReader reader = new ClassReader(is);
						TypeCollector visitor = new TypeCollector();
						reader.accept(visitor, 0);
						if (visitor.isComponentAnnotationFound()) {
							String name = current + child.getName();
							name = name.substring(0, name.lastIndexOf("."));
							getLog().debug("Found type " + name);
							list.add(name);
						}
					} catch (FileNotFoundException e) {
						throw new MojoExecutionException("Error loading file "
								+ child.getAbsolutePath(), e);
					} catch (IOException e) {
						throw new MojoExecutionException(
								"Error processing file "
										+ child.getAbsolutePath(), e);
					}
				}
			}
		}
	}

}
